/** @file system.h
*   @brief System Driver Header File
*   @date 15.January.2012
*   @version 3.00.00
*   
*   This file contains:
*   - Definitions
*   - Types
*   .
*   which are relevant for the System driver.
*/

/* (c) Texas Instruments 2012, All rights reserved. */

#ifndef __SYS_SYSTEM_H__
#define __SYS_SYSTEM_H__

//#include "*** include variable HDRCOMMON not defined ***"
#include "sys_common.h"
/* USER CODE BEGIN (0) */
/* USER CODE END */


/* System General Definitions */

/** @enum systemClockSource
*   @brief Alias names for clock sources
*
*   This enumeration is used to provide alias names for the clock sources:
*     - Oscillator
*     - Pll
*     - 32 kHz Oscillator
*     - External
*     - Low Power Oscillator Low
*     - Low Power Oscillator High
*     - Synchronous VCLK1
*/

enum systemClockSource
{
    SYS_OSC      = 0, /**< Alias for oscillator clock Source                */
    SYS_PLL      = 1, /**< Alias for Pll clock Source                       */
    SYS_O32      = 2, /**< Alias for 32 kHz oscillator clock Source         */
    SYS_EXTERNAL = 3, /**< Alias for external clock Source                  */
    SYS_LPO_LOW  = 4, /**< Alias for low power oscillator low clock Source  */
    SYS_LPO_HIGH = 5, /**< Alias for low power oscillator high clock Source */
    SYS_VCLK     = 9  /**< Alias for synchronous VCLK1 clock Source         */
};

/** @enum systemClockDomain
*   @brief Alias names for clock domain
*
*   This enumeration is used to provide alias names for the clock domains:
*     - GCLK
*     - ACCLK1
*     
*/

enum systemClockDomain
{
    SYS_GCLK     = 8,   /**< Alias for GCLK1 clock domain     */
    SYS_AVCLK1   = 11   /**< Alias for AVCLK1 clock domain    */
};



/** @def SYS_PRE1
*   @brief Alias name for RTI1CLK PRE clock source
*
*   This is an alias name for the RTI1CLK pre clock source.
*   This can be either:
*     - Oscillator
*     - Pll
*     - 32 kHz Oscillator
*     - External
*     - Low Power Oscillator Low
*     - Low Power Oscillator High
*/
#define SYS_PRE1 SYS_PLL

/** @def SYS_PRE2
*   @brief Alias name for RTI2CLK pre clock source
*
*   This is an alias name for the RTI2CLK pre clock source.
*   This can be either:
*     - Oscillator
*     - Pll
*     - 32 kHz Oscillator
*     - External
*     - Low Power Oscillator Low
*     - Low Power Oscillator High
*/
#define SYS_PRE2 SYS_PLL

/* USER CODE BEGIN (1) */
/* USER CODE END */

/* System Register Frame 1 Definition */

typedef volatile struct systemBase
{
    unsigned SYSPC1;                 /* 0x0000 */
    unsigned SYSPC2;                 /* 0x0004 */
    unsigned SYSPC3;                 /* 0x0008 */
    unsigned SYSPC4;                 /* 0x000C */
    unsigned SYSPC5;                 /* 0x0010 */
    unsigned SYSPC6;                 /* 0x0014 */
    unsigned SYSPC7;                 /* 0x0018 */
    unsigned SYSPC8;                 /* 0x001C */
    unsigned SYSPC9;                 /* 0x0020 */
    unsigned SSWPLL1;                /* 0x0024 */
    unsigned SSWPLL2;                /* 0x0028 */
    unsigned SSWPLL3;                /* 0x002C */
    unsigned CSDIS;                  /* 0x0030 */
    unsigned CSDISSET;               /* 0x0034 */
    unsigned CSDISCLR;               /* 0x0038 */
    unsigned CDDIS;                 /* 0x003C */
    unsigned CDDISSET;              /* 0x0040 */
    unsigned CDDISCLR;              /* 0x0044 */
    unsigned GHVSRC;                 /* 0x0048 */
    unsigned VCLKASRC;               /* 0x004C */
    unsigned RCLKSRC;                /* 0x0050 */
    unsigned CSVSTAT;                /* 0x0054 */
    unsigned MSTGCR;                 /* 0x0058 */
    unsigned MINITGCR;               /* 0x005C */
    unsigned MSINENA;                /* 0x0060 */
    unsigned MSTFAIL;                /* 0x0064 */
    unsigned MSTCGSTAT;              /* 0x0068 */
    unsigned MINISTAT;               /* 0x006C */
    unsigned PLLCTL1;                /* 0x0070 */
    unsigned PLLCTL2;                /* 0x0074 */
    unsigned UERFLAG;                /* 0x0078 */
    unsigned DIEIDL;                 /* 0x007C */
    unsigned DIEIDH;                 /* 0x0080 */
    unsigned VRCTL;                  /* 0x0084 */
    unsigned LPOMONCTL;              /* 0x0088 */
    unsigned CLKTEST;                /* 0x008C */
    unsigned DFTCTRLREG1;            /* 0x0090 */
    unsigned DFTCTRLREG2;            /* 0x0094 */
    unsigned : 32U;                  /* 0x0098 */
    unsigned : 32U;                  /* 0x009C */
    unsigned GPREG1;                 /* 0x00A0 */
    unsigned BTRMSEL;                /* 0x00A4 */
    unsigned IMPFASTS;               /* 0x00A8 */
    unsigned IMPFTADD;               /* 0x00AC */
    unsigned SSISR1;                 /* 0x00B0 */
    unsigned SSISR2;                 /* 0x00B4 */
    unsigned SSISR3;                 /* 0x00B8 */
    unsigned SSISR4;                 /* 0x00BC */
    unsigned RAMGCR;                 /* 0x00C0 */
    unsigned BMMCR1;                 /* 0x00C4 */
    unsigned BMMCR2;                 /* 0x00C8 */
    unsigned MMUGCR;                 /* 0x00CC */
#ifdef _little_endian__
    unsigned        : 8U;            /* 0x00D0 */
    unsigned PENA   : 1U;            /* 0x00D0 */
    unsigned        : 7U;            /* 0x00D0 */
    unsigned VCLKR  : 4U;            /* 0x00D0 */
    unsigned        : 4U;            /* 0x00D0 */
    unsigned VCLK2R : 4U;            /* 0x00D0 */
    unsigned        : 4U;            /* 0x00D0 */
#else
    unsigned        : 4U;            /* 0x00D0 */
    unsigned VCLK2R : 4U;            /* 0x00D0 */
    unsigned        : 4U;            /* 0x00D0 */
    unsigned VCLKR  : 4U;            /* 0x00D0 */
    unsigned        : 7U;            /* 0x00D0 */
    unsigned PENA   : 1U;            /* 0x00D0 */
    unsigned        : 8U;            /* 0x00D0 */
#endif
    unsigned : 32U;                  /* 0x00D4 */
    unsigned DSPGCR;                 /* 0x00D8 */
    unsigned DEVCR1;                 /* 0x00DC */
    unsigned SYSECR;                 /* 0x00E0 */
    unsigned SYSESR;                 /* 0x00E4 */
    unsigned ITIFLAG;                /* 0x00E8 */
    unsigned GBLSTAT;                /* 0x00EC */
    unsigned DEV;                    /* 0x00F0 */
    unsigned SSIVEC;                 /* 0x00F4 */
    unsigned SSIF;                   /* 0x00F8 */
} systemBASE_t;

/** @def systemREG
*   @brief System Register Frame 1 Pointer
*
*   This pointer is used by the system driver to access the system frame 1 registers.
*/

#define systemREG ((systemBASE_t *)0xFFFFFF00U)

/* USER CODE BEGIN (2) */
/* USER CODE END */


/**************************************/
/*    Defining the MPU regions        */
/**************************************/
#define REGION0			0x0
#define REGION1 			0x1
#define REGION2			0x2
#define REGION3			0x3
#define REGION4			0x4
#define REGION5			0x5	
#define REGION6			0x6
#define REGION7			0x7

/*************************************************/
/*    Defining the MPU access permissions        */
/*************************************************/

#define AP_EXE_NOACCESS		0x00000000
#define AP_EXE_READWRITE	0x03000000
#define AP_EXE_READONLY		0x07000000

#define AP_NON_EXE_NOACCESS	0x10000000
#define AP_NON_EXE_READWRITE	0x13000000
#define AP_NON_EXE_READONLY	0x17000000

/**************************************/
/*    Defining the MPU region size    */
/**************************************/

#define Size_32B			0x00000008
#define Size_64B			0x0000000A
#define Size_128B			0x0000000C
#define Size_256B	 		0x0000000E
#define Size_512B			0x00000010
#define Size_1KB			0x00000012
#define Size_2KB			0x00000014
#define Size_4KB			0x00000016
#define Size_8KB			0x00000018
#define Size_16KB			0x0000001A
#define Size_32KB			0x0000001C
#define Size_64KB			0x0000001E
#define Size_128KB		0x00000020
#define Size_256KB		0x00000022
#define Size_512KB		0x00000024
#define Size_1MB 			0x00000026
#define Size_2MB		   	0x00000028
#define Size_4MB			0x0000002A
#define Size_8MB			0x0000002C
#define Size_16MB			0x0000002E
#define Size_32MB			0x00000030
#define Size_64MB			0x00000032
#define Size_128MB		0x00000034
#define Size_256MB		0x00000036
#define Size_512MB		0x00000038
#define Size_1GB			0x0000003A
#define Size_2GB			0x0000003C
#define Size_4GB			0x0000003E

/* M3 Core Register Frame Definition */

typedef volatile struct m3coreBase
{
    uint32_t MASTERCTRL;      /* 0x0000        */
    uint32_t INTCONTRTYPE;    /* 0x0004        */
    uint32_t : 32;            /* 0x0008        */
    uint32_t : 32;            /* 0x000C        */
    uint32_t SYSTICKCTRLSR;   /* 0x0010        */
    uint32_t SYSTICKRELOAD;   /* 0x0014        */
    uint32_t SYSTICKCURRENT;  /* 0x0018        */
    uint32_t SYSTICKCAL;      /* 0x001C        */
    uint32_t Reserved_0[56];  /* 0x0020-0x00FF */
    uint32_t SETENA[8];       /* 0x0100-0x011F */
    uint32_t Reserved_1[24];  /* 0x0120-0x017F */
    uint32_t CLRENA[8];       /* 0x0180-0x019F */
    uint32_t Reserved_2[24];  /* 0x01A0-0x01FF */
    uint32_t SETPEND[8];      /* 0x0200-0x021F */
    uint32_t Reserved_3[24];  /* 0x0220-0x027F */
    uint32_t CLRPEND[8];      /* 0x0280-0x029F */
    uint32_t Reserved_4[24];  /* 0x02A0-0x02FF */
    uint32_t ACTIVE[8];       /* 0x0300-0x031F */
    uint32_t Reserved_5[24];  /* 0x0320-0x037F */
    uint32_t Reserved_6[32];  /* 0x0380-0x03FF */
    uint32_t INTPRIO[61];     /* 0x0400-0x07EF */
    uint32_t Reserved_7[515]; /* 0x07F0-0x0CFF */
    uint32_t CPUID;           /* 0x0D00        */
    uint32_t INTCTRL;         /* 0x0D04        */
    uint32_t VTBLOFF;         /* 0x0D08        */
    uint32_t AIRCR;           /* 0x0D0C        */
    uint32_t SYSCTRL;         /* 0x0D10        */
    uint32_t CONFCTRL;        /* 0x0D14        */
    uint32_t SYSPRIO[3];      /* 0x0D18-0x0D23 */
    uint32_t SYSHCTRL;        /* 0x0D24        */
    uint32_t CONFFAULTSR;     /* 0x0D28        */
    uint32_t HARDFAULTSR;     /* 0x0D2C        */
    uint32_t DEBUGFAULTSR;    /* 0x0D30        */
    uint32_t MEMMANAGEADDR;   /* 0x0D34        */
    uint32_t BUSFAULTADDR;    /* 0x0D38        */
    uint32_t AUXFAULTADDR;    /* 0x0D3C        */
    uint32_t PROCFEATURE0;    /* 0x0D40        */
    uint32_t PROCFEATURE1;    /* 0x0D44        */
    uint32_t DEBUGFEATURE;    /* 0x0D48        */
    uint32_t AUXFEATURE;      /* 0x0D4C        */
    uint32_t MEMMODFEATURE0;  /* 0x0D50        */
    uint32_t MEMMODFEATURE1;  /* 0x0D54        */
    uint32_t MEMMODFEATURE2;  /* 0x0D58        */
    uint32_t MEMMODFEATURE3;  /* 0x0D5C        */
    uint32_t ISAFEATURE0;     /* 0x0D60        */
    uint32_t ISAFEATURE1;     /* 0x0D64        */
    uint32_t ISAFEATURE2;     /* 0x0D68        */
    uint32_t ISAFEATURE3;     /* 0x0D6C        */
    uint32_t ISAFEATURE4;     /* 0x0D70        */
    uint32_t ISAFEATURE5;     /* 0x0D74        */
    uint32_t : 32;            /* 0x0D78        */
    uint32_t : 32;            /* 0x0D7C        */
    uint32_t : 32;            /* 0x0D80        */
    uint32_t : 32;            /* 0x0D84        */
    uint32_t COPROCACESS;     /* 0x0D88        */
    uint32_t : 32;            /* 0x0D8C        */
    uint32_t MPUTR;           /* 0x0D90        */
    uint32_t MPUCR;           /* 0x0D94        */
    uint32_t MPURNR;          /* 0x0D98        */
    uint32_t MPURBAR0;        /* 0x0D9C        */
    uint32_t MPURASR0;        /* 0x0DA0        */
    uint32_t MPURBAR1;        /* 0x0DA4        */
    uint32_t MPURASR1;        /* 0x0DA8        */
    uint32_t MPURBAR2;        /* 0x0DAC        */
    uint32_t MPURASR2;        /* 0x0DB0        */
    uint32_t MPURBAR3;        /* 0x0DB4        */
    uint32_t MPURASR3;        /* 0x0DB8        */
    uint32_t Reserved_8[15];  /* 0x0DBC-0x0DEF */
    uint32_t DHCSR;           /* 0x0DF0        */
    uint32_t DCRSR;           /* 0x0DF4        */
    uint32_t DCRDR;           /* 0x0DF8        */
    uint32_t DEMCR;           /* 0x0DFC        */
    uint32_t Reserved_9[64];  /* 0x0E00-0x0EFF */
    uint32_t SWTRIGINT;       /* 0x0F00        */
    uint32_t Reserved_10[51]; /* 0x0F04-0x0FCF */
    uint32_t PERID[8];        /* 0x0FD0-0x0FEF */
    uint32_t CID[4];          /* 0x0FF0-0x0FFF */
}m3coreBASE_t;

/** @def M3CORE
*   @brief M3 Core Register Frame Definition
*
*   This pointer is used by the system driver to access the M3 Core Registers
*/

#define M3CORE ((m3coreBASE_t *)0xE000E000)


/* PCR Register Definition */

typedef volatile struct pcrBase
{
    unsigned PMPROTSET0;    /* 0x0000 */
    unsigned PMPROTSET1;    /* 0x0004 */
    unsigned : 32U;         /* 0x0008 */
    unsigned : 32U;         /* 0x000C */
    unsigned PMPROTCLR0;    /* 0x0010 */
    unsigned PMPROTCLR1;    /* 0x0014 */
    unsigned : 32U;         /* 0x0018 */
    unsigned : 32U;         /* 0x001C */
    unsigned PPROTSET0;     /* 0x0020 */
    unsigned PPROTSET1;     /* 0x0024 */
    unsigned PPROTSET2;     /* 0x0028 */
    unsigned PPROTSET3;     /* 0x002C */
    unsigned : 32U;         /* 0x0030 */
    unsigned : 32U;         /* 0x0034 */
    unsigned : 32U;         /* 0x0038 */
    unsigned : 32U;         /* 0x003C */
    unsigned PPROTCLR0;     /* 0x0040 */
    unsigned PPROTCLR1;     /* 0x0044 */
    unsigned PPROTCLR2;     /* 0x0048 */
    unsigned PPROTCLR3;     /* 0x004C */
    unsigned : 32U;         /* 0x0050 */
    unsigned : 32U;         /* 0x0054 */
    unsigned : 32U;         /* 0x0058 */
    unsigned : 32U;         /* 0x005C */
    unsigned PCSPWRDWNSET0; /* 0x0060 */
    unsigned PCSPWRDWNSET1; /* 0x0064 */
    unsigned : 32U;         /* 0x0068 */
    unsigned : 32U;         /* 0x006C */
    unsigned PCSPWRDWNCLR0; /* 0x0070 */
    unsigned PCSPWRDWNCLR1; /* 0x0074 */
    unsigned : 32U;         /* 0x0078 */
    unsigned : 32U;         /* 0x007C */
    unsigned PSPWRDWNSET0;  /* 0x0080 */
    unsigned PSPWRDWNSET1;  /* 0x0084 */
    unsigned PSPWRDWNSET2;  /* 0x0088 */
    unsigned PSPWRDWNSET3;  /* 0x008C */
    unsigned : 32U;         /* 0x0090 */
    unsigned : 32U;         /* 0x0094 */
    unsigned : 32U;         /* 0x0098 */
    unsigned : 32U;         /* 0x009C */
    unsigned PSPWRDWNCLR0;  /* 0x00A0 */
    unsigned PSPWRDWNCLR1;  /* 0x00A4 */
    unsigned PSPWRDWNCLR2;  /* 0x00A8 */
    unsigned PSPWRDWNCLR3;  /* 0x00AC */
} pcrBASE_t;

/** @def pcrREG
*   @brief PCR Register Frame Pointer
*
*   This pointer is used by the system driver to access the PCR registers.
*/

#define pcrREG ((pcrBASE_t *)0xFFFFE000U)

/* USER CODE BEGIN (4) */
/* USER CODE END */


/* FlashW General Definitions */


/** @enum flashWPowerModes
*   @brief Alias names for flash bank power modes
*
*   This enumeration is used to provide alias names for the flash bank power modes:
*     - sleep
*     - standby
*     - active
*/

enum flashWPowerModes
{
    SYS_SLEEP   = 0U, /**< Alias for flash bank power mode sleep   */
    SYS_STANDBY = 1U, /**< Alias for flash bank power mode standby */
    SYS_ACTIVE  = 3U  /**< Alias for flash bank power mode active  */
};

/* USER CODE BEGIN (5) */
/* USER CODE END */

/** @struct flashWBase
*   @brief Flash Wrapper Register Frame Definition
*
*   This type is used to access the Flash Wrapper Registers.
*/
/** @typedef flashWBASE_t
*   @brief Flash Wrapper Register Frame Type Definition
*
*   This type is used to access the Flash Wrapper Registers.
*/
typedef volatile struct flashWBase
{
    unsigned FRDCNTL;     /* 0x0000 */
    unsigned FSPRD;       /* 0x0004 */
    unsigned FEDACCTRL1;  /* 0x0008 */
    unsigned FEDACCTRL2;  /* 0x000C */
    unsigned FCORERRCNT;  /* 0x0010 */
    unsigned FCORERRADD;  /* 0x0014 */
    unsigned FCORERRPOS;  /* 0x0018 */
    unsigned FEDACSTATUS; /* 0x001C */
    unsigned FUNCERRADD;  /* 0x0020 */
    unsigned FEDACSDIS;   /* 0x0024 */
    unsigned FPRIMADDTAG; /* 0x0028 */
    unsigned FREDUADDTAG; /* 0x002C */
    unsigned FBPROT;      /* 0x0030 */
    unsigned FBSE;        /* 0x0034 */
    unsigned FBBUSY;      /* 0x0038 */
    unsigned FBAC;        /* 0x003C */
    unsigned FBFALLBACK;  /* 0x0040 */
    unsigned FBPRDY;      /* 0x0044 */
    unsigned FPAC1;       /* 0x0048 */
    unsigned FPAC2;       /* 0x004C */
    unsigned FMAC;        /* 0x0050 */
    unsigned FMSTAT;      /* 0x0054 */
    unsigned FEMUDMSW;    /* 0x0058 */
    unsigned FEMUDLSW;    /* 0x005C */
    unsigned FEMUECC;     /* 0x0060 */
    unsigned FLOCK;       /* 0x0064 */
    unsigned FEMUADDR;    /* 0x0068 */
    unsigned FDIAGCTRL;   /* 0x006C */
    unsigned FRAWDATAH;   /* 0x0070 */
    unsigned FRAWDATAL;   /* 0x0074 */
    unsigned FRAWECC;     /* 0x0078 */
    unsigned FPAROVR;     /* 0x007C */
    unsigned FVREADCT;    /* 0x0080 */
    unsigned FVHVCT1;     /* 0x0084 */
    unsigned FVHVCT2;     /* 0x0088 */
    unsigned FVNVCT;      /* 0x008C */
    unsigned FVPPCT;      /* 0x0090 */
    unsigned FVWLCT;      /* 0x0094 */
    unsigned FEFUSE;      /* 0x0098 */
    unsigned : 32U;       /* 0x009C */
    unsigned : 32U;       /* 0x00A0 */
    unsigned : 32U;       /* 0x00A4 */
    unsigned : 32U;       /* 0x00A8 */
    unsigned : 32U;       /* 0x00AC */
    unsigned : 32U;       /* 0x00B0 */
    unsigned : 32U;       /* 0x00B4 */
    unsigned : 32U;       /* 0x00B8 */
    unsigned : 32U;       /* 0x00BC */
    unsigned FEDACSDIS2;  /* 0x00C0 */
    unsigned : 32U;       /* 0x00C4 */
    unsigned : 32U;       /* 0x00C8 */
    unsigned : 32U;       /* 0x00CC */
    unsigned : 32U;       /* 0x00D0 */
    unsigned : 32U;       /* 0x00D4 */
    unsigned : 32U;       /* 0x00D8 */
    unsigned : 32U;       /* 0x00DC */
    unsigned : 32U;       /* 0x00E0 */
    unsigned : 32U;       /* 0x00E4 */
    unsigned : 32U;       /* 0x00E8 */
    unsigned : 32U;       /* 0x00EC */
    unsigned : 32U;       /* 0x00F0 */
    unsigned : 32U;       /* 0x00F4 */
    unsigned : 32U;       /* 0x00F8 */
    unsigned : 32U;       /* 0x00FC */
    unsigned FBSTROBES;   /* 0x0100 */
    unsigned FPSTROBES;   /* 0x0104 */
    unsigned FBMODE;      /* 0x0108 */
    unsigned FTCR;        /* 0x010C */
    unsigned FADDR;       /* 0x0110 */
    unsigned FWRITE;      /* 0x0114 */
    unsigned FCBITSEL;    /* 0x0118 */
    unsigned FTCTRL;      /* 0x011C */
    unsigned FWPWRITE0;   /* 0x0120 */
    unsigned FWPWRITE1;   /* 0x0124 */
    unsigned FWPWRITE2;   /* 0x0128 */
    unsigned FWPWRITE3;   /* 0x012C */
    unsigned FWPWRITE4;   /* 0x0130 */
} flashWBASE_t;

/** @def flashWREG
*   @brief Flash Wrapper Register Frame Pointer
*
*   This pointer is used by the system driver to access the flash wrapper registers.
*/
#define flashWREG ((flashWBASE_t *)(0xFFF87000U))


/* System Interface Functions */

extern void systemInit(void);
void systemPowerDown(unsigned mode);

/* USER CODE BEGIN (8) */
/* USER CODE END */


#endif
