/* (c) Texas Instruments 2012, All rights reserved. */

#include "sys_lbist.h"
#include "system.h"
#include "gio.h"
#include "het.h"

/** @fn void _lbistTest_(void)
*   @brief LBIST test routine
*   @note This function defines LBIST test routine at startup.
*
*   The CPU LBIST test is specific to the microcontroller.
*   The complete CPU test is run only once on startup when enabled.
*   There will be a cpu reset after test completion.
*   After reset the LBIST test will be skipped in the boot routine before proceeding to main.
*   ----------------------------------------------------------------------------------------
*   !!!!  WARNING   !!!!
*   The user is not supposed to modify this LBIST code. If this test gets into an endless loop
*   the CPU will not be able to establish communication with JTAG .
*   ----------------------------------------------------------------------------------------
*   Status:               Pin     Pin    Pin      Reg
*                         HET2,   HET1,  HET0     lbist_stat
*   Test Start         :  ON      OFF    OFF      0x4
*   Test Complete PASS :  ON      ON     ON       0x7
*   Test Complete FAIL :  OFF     ON     OFF      0x2
*   Test Incomplete    :  OFF     OFF    ON       0x1 
*   ----------------------------------------------------------------------------------------
*/

#define STC_Interval   24
#define STC_TimeOut    0xFFFFFFFF

#define STC_ON         0xA
#define LBIST_START    0x4
#define LBIST_PASS     0x7
#define LBIST_FAIL     0x2
#define LBIST_NC       0x1
#define wait1          0x100
#define wait2          0xFFFF
 
#define LBIST_CLK_DIV_REG     (uint32_t *) 0xFFFFE108  /* CPU Logic BIST Clock Prescaler */
#define LBIST_CLK_DIV_RATIO    0x1                         /* Divide  ratio for this test    */   

#if 0    /* Define Only if LBIST Test Enabled */

void lbistClockSet(void);


void _lbistTest_(void)
{
   unsigned lbist_stat  = 0;
   unsigned lbist_delay = 0;
   unsigned led_delay   = 0;
  
   lbist_stat  = lbist_stat;
   lbist_delay = wait1;
   led_delay   = wait2;


    /* Step 1 : Configure Pins to Flag Test status  */
	  
    /* GIO A4 -> Check this PIN and proceed if Low */
    gioREG->GCR0      = 0x1;
    gioPORTA->DIR    |= 0x00000010;
    gioPORTA->PULDIS |= 0x00000010;
    
    
    /* HET PINS 0,1,2 */
    hetREG->DIR    = 0x7;
    hetREG->PULDIS = 0x7;
    hetREG->DOUT   = 0x0;
    

    /* Step 2 : Check CPU Status */

    if((systemREG->SYSESR & 0x20) == 0x20 )
    {

      /* Check for GIOA4 Low to start CPU LBIST Test */
      while(lbist_delay--);
      while((gioPORTA->DIN & 0x10) == 0x10); /*     Wait until pin low                 */
                                             /* !!! Warning - Do not remove this check */

      systemREG->SYSESR == 0x20;             /* Clear reset flag   */
    
      switch(lbistREG->STCGSTAT & 0x3)
      { 
	    case 0: /* Test not Started */
        lbist_stat = 0x0;
        lbistNotification(lbist_stat);
		break;
		
        case 1: /* Test Completed and Passed */
        lbist_stat     = LBIST_PASS;
        hetREG->DOUT   = LBIST_PASS;
		break;

        case 2:	/* Test Not Completed */
	    //lbistREG->STCGSTAT = lbistREG->STCGSTAT;        
        lbist_stat     = LBIST_NC;
        hetREG->DOUT   = LBIST_NC;  
        lbistNotification(lbist_stat);
		break;		

        case 3:	/* Test Completed and Failed*/
	    //lbistREG->STCGSTAT = lbistREG->STCGSTAT;        
        lbist_stat     = LBIST_FAIL;  
        hetREG->DOUT   = LBIST_FAIL;        
        lbistNotification(lbist_stat);      
		break;			
	     
      }
    }
    else
    {
 
      /*  Step 3 : Check for GIOA4 Low to start CPU LBIST Test */
      while(lbist_delay--);
      while((gioPORTA->DIN & 0x10) == 0x10); /*    Wait until pin low                 */
                                             /*!!! Warning - Do not remove this check */

	  /* Start LBIST TEST     */
	  
     hetREG->DOUT  = LBIST_START;  /* Indicate Test Start */
     lbist_stat    = LBIST_START;
     while(led_delay--);               
      
     /* Flash Wait States */ 
     *(int *) 0xfff87008  = 0x0005;
  	 *(int *) 0xfff87000  = 0x411;

     /* Clock Configurations for LBIST Test */      
     lbistClockSet(); /* 80MHz VCLK2 for 16MHz Oscin */
  

     /* LBIST Initialization */

     lbistREG->STCGCR0 = STC_Interval <<16U | 0x1;
     lbistREG->STCTPR  = STC_TimeOut;


 	  asm("	nop");                   
	  asm("	nop");
	  asm("	nop");
	  asm("	nop");

      /* Step 6: Start LBIST Test  */
      lbistREG->STCGCR1 = STC_ON;
      lbistREG->STCGCR1;
    
 	  asm("	nop");                   /* Allow 16 clock cycles */
	  asm("	nop");
	  asm("	nop");
	  asm("	nop");
 	  asm("	nop");
	  asm("	nop");
	  asm("	nop");
	  asm("	nop");
 	  asm("	nop");
	  asm("	nop");
	  asm("	nop");
	  asm("	nop");	  	  
 	  asm("	nop");
	  asm("	nop");
	  asm("	nop");
	  asm("	nop");
	  asm("	WFI");
	  

      while(1);          /* Wait until test finishes with a reset ... */

    }

}


void lbistClockSet(void)
{
    systemREG->PLLCTL1  =  0x21046300; 
    systemREG->PLLCTL2  =  0x3FC0023D;
    while ((systemREG->CSVSTAT & 0x2) != 0x2);
    
    /* VCLK = HCLK = 80 MHz */
    *(int *) 0xffffffd0 = 0x00000100;
     
    /* LBIST CLOCK @ HCLK/2 = 40 MHz */
    *LBIST_CLK_DIV_REG = (LBIST_CLK_DIV_RATIO << 24U); 
}


void lbistNotification(unsigned notification)
{
 notification = notification;
/* USER CODE BEGIN (0) */
/* USER CODE END */
}



#endif

/* - End of sys_lbist.c -------------------------------------------------------------------*/







